package mp3sorter;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSlider;
import javax.swing.KeyStroke;

public class MainWindow extends
		JFrame
{
	private static final long serialVersionUID = 30510642651771066L;

	private Mp3PlayerController controller = null;

	private int outsidePadding = 20;
	private Color backgroundColor = new Color(
			235,
			235,
			255);

	public Label authorLabel = null;
	public Label albumLabel = null;
	public Label titleLabel = null;

	private JButton shuffleButton;
	private JButton loopButton;
	private JButton pauseButton;
	private JSlider seekSlider;

	private Long keyPressTime = null;

	public MainWindow() {
		super(
				"Aerobush MP3 Sorter");
		this.setResizable(false);
		this.getContentPane().setBackground(
				backgroundColor);
		this.getContentPane().setLayout(
				new GridBagLayout());

		initID3();
		initRatingButtons();
		initPlaybackButtons();
		setKeyBindings();

		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		controller = new Mp3PlayerController(
				this);
		this.pack();
		this.setVisible(true);
	}

	private void initID3() {
		authorLabel = new Label(
				"Artist: ");
		authorLabel.setAlignment(Label.CENTER);
		albumLabel = new Label(
				"Album: ");
		albumLabel.setAlignment(Label.CENTER);
		titleLabel = new Label(
				"Title: ");
		titleLabel.setAlignment(Label.CENTER);

		JPanel tagPanel = new JPanel();
		tagPanel.setBackground(new Color(
				255,
				255,
				255));
		tagPanel.setBorder(BorderFactory.createLineBorder(
				new Color(
						0,
						0,
						255),
				3));

		tagPanel.setLayout(new GridBagLayout());
		GridBagConstraints gridBag = new GridBagConstraints();
		gridBag.gridx = 0;
		gridBag.gridy = 0;
		tagPanel.add(
				authorLabel,
				gridBag);
		gridBag.gridy = 1;
		tagPanel.add(
				albumLabel,
				gridBag);
		gridBag.gridy = 2;
		tagPanel.add(
				titleLabel,
				gridBag);

		gridBag = new GridBagConstraints();
		gridBag.gridx = 0;
		gridBag.gridy = 0;
		gridBag.insets = new Insets(
				outsidePadding,
				outsidePadding,
				50,
				outsidePadding);
		this.getContentPane().add(
				tagPanel,
				gridBag);
	}

	private void initRatingButtons() {
		JButton noButton = new JButton(
				"No");
		noButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.relocateFile(0);
			}
		});
		noButton.setFocusable(false);
		noButton.setRolloverEnabled(true);
		noButton.setEnabled(true);

		JButton maybeButton = new JButton(
				"Maybe");
		maybeButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.relocateFile(1);
			}
		});
		maybeButton.setFocusable(false);
		maybeButton.setRolloverEnabled(true);
		maybeButton.setEnabled(true);

		JButton yesButton = new JButton(
				"Yes");
		yesButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.relocateFile(2);
			}
		});
		yesButton.setFocusable(false);
		yesButton.setRolloverEnabled(true);
		yesButton.setEnabled(true);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setBackground(backgroundColor);
		buttonPanel.setLayout(new GridBagLayout());
		GridBagConstraints gridBag = new GridBagConstraints();

		gridBag.insets = new Insets(
				0,
				outsidePadding,
				0,
				20);
		gridBag.gridx = 0;
		buttonPanel.add(
				noButton,
				gridBag);
		gridBag.insets = new Insets(
				0,
				0,
				0,
				20);
		gridBag.gridx = 1;
		buttonPanel.add(
				maybeButton,
				gridBag);
		gridBag.gridx = 2;
		gridBag.insets = new Insets(
				0,
				0,
				0,
				outsidePadding);
		buttonPanel.add(
				yesButton,
				gridBag);

		gridBag = new GridBagConstraints();
		gridBag.gridx = 0;
		gridBag.gridy = 1;
		gridBag.insets = new Insets(
				0,
				0,
				50,
				0);
		this.getContentPane().add(
				buttonPanel,
				gridBag);
	}

	private synchronized void buttonUpdate(
			JButton button,
			String offIcon,
			String onIcon ) {
		if (button.getName().matches(
				"disabled")) {
			button.setIcon(new ImageIcon(
					"./profile/skin/" + onIcon));
			button.setName("enabled");
		}
		else {
			button.setIcon(new ImageIcon(
					"./profile/skin/" + offIcon));
			button.setName("disabled");
		}
	}

	private void initPlaybackButtons() {
		seekSlider = new JSlider(
				JSlider.HORIZONTAL,
				0,
				1000,
				0);
		seekSlider.addMouseListener(new MouseAdapter() {
			public void mouseReleased(
					MouseEvent e ) {
				seekSlider.setValue((1000 * e.getX()) / seekSlider.getSize().width);
				controller.seekTo(seekSlider.getValue());
			}
		});

		shuffleButton = new JButton(
				new ImageIcon(
						"./profile/skin/shuffle_off.png"));
		shuffleButton.setName("disabled");
		shuffleButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.toggleShuffle();
				buttonUpdate(
						shuffleButton,
						"shuffle_off.png",
						"shuffle_on.png");
			}
		});
		shuffleButton.setFocusable(false);
		shuffleButton.setRolloverEnabled(true);
		shuffleButton.setEnabled(true);

		JButton folderButton = new JButton(
				new ImageIcon(
						"./profile/skin/folder.png"));
		folderButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.changeDir();
			}
		});
		folderButton.setFocusable(false);
		folderButton.setRolloverEnabled(true);
		folderButton.setEnabled(true);

		loopButton = new JButton(
				new ImageIcon(
						"./profile/skin/loop_off.png"));
		loopButton.setName("disabled");
		loopButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.toggleLoop();
				buttonUpdate(
						loopButton,
						"loop_off.png",
						"loop_on.png");
			}
		});
		loopButton.setFocusable(false);
		loopButton.setRolloverEnabled(true);
		loopButton.setEnabled(true);

		final JSlider volumeSlider = new JSlider(
				JSlider.VERTICAL,
				0,
				100,
				100);
		final JPopupMenu popup = new JPopupMenu();
		popup.add(volumeSlider);
		final JButton volumeButton = new JButton(
				new ImageIcon(
						"./profile/skin/volume.png"));
		volumeButton.setFocusable(false);
		volumeButton.setRolloverEnabled(true);
		volumeButton.setEnabled(true);
		volumeButton.addMouseListener(new MouseAdapter() {
			public void mousePressed(
					MouseEvent e ) {
				popup.show(
						volumeButton,
						volumeButton.getWidth() - 20,
						-200);
			}
		});
		volumeSlider.addMouseListener(new MouseAdapter() {
			public void mouseReleased(
					MouseEvent e ) {
				volumeSlider.setValue(100 - (100 * e.getY()) / volumeSlider.getSize().height);
				controller.setVolume(volumeSlider.getValue());
				popup.setVisible(false);
			}
		});

		JButton previousButton = new JButton(
				new ImageIcon(
						"./profile/skin/previous.png"));
		previousButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.previousMp3();
			}
		});
		previousButton.setFocusable(false);
		previousButton.setRolloverEnabled(true);
		previousButton.setEnabled(true);

		JButton rewindButton = new JButton(
				new ImageIcon(
						"./profile/skin/rewind.png"));
		rewindButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.rewindMp3();
			}
		});
		rewindButton.setFocusable(false);
		rewindButton.setRolloverEnabled(true);
		rewindButton.setEnabled(true);

		pauseButton = new JButton(
				new ImageIcon(
						"./profile/skin/pause.png"));
		pauseButton.setName("disabled");
		pauseButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.pauseMp3();
				buttonUpdate(
						pauseButton,
						"pause.png",
						"play.png");
			}
		});
		pauseButton.setFocusable(false);
		pauseButton.setRolloverEnabled(true);
		pauseButton.setEnabled(true);

		JButton fastforwardButton = new JButton(
				new ImageIcon(
						"./profile/skin/fastforward.png"));
		fastforwardButton.addMouseListener(new MouseAdapter() {
			long mousePressTime = 0;

			public void mousePressed(
					MouseEvent e ) {
				mousePressTime = System.currentTimeMillis();
			}

			public void mouseReleased(
					MouseEvent e ) {
				controller.fastForwardMp3(System.currentTimeMillis() - mousePressTime);
			}
		});
		fastforwardButton.setFocusable(false);
		fastforwardButton.setRolloverEnabled(true);
		fastforwardButton.setEnabled(true);

		JButton nextButton = new JButton(
				new ImageIcon(
						"./profile/skin/next.png"));
		nextButton.addActionListener(new ActionListener() {
			public void actionPerformed(
					ActionEvent e ) {
				controller.nextMp3(new Exception(
						"Next button was pressed."));
			}
		});
		nextButton.setFocusable(false);
		nextButton.setRolloverEnabled(true);
		nextButton.setEnabled(true);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setBackground(backgroundColor);
		buttonPanel.setLayout(new GridBagLayout());
		GridBagConstraints gridBag = new GridBagConstraints();

		gridBag.gridx = 0;
		gridBag.gridy = 0;
		gridBag.gridwidth = 5;
		gridBag.insets = new Insets(
				0,
				0,
				10,
				0);
		buttonPanel.add(
				seekSlider,
				gridBag);

		gridBag.gridx = 1;
		gridBag.gridy = 1;
		gridBag.gridwidth = 1;
		gridBag.insets = new Insets(
				0,
				0,
				10,
				0);
		buttonPanel.add(
				shuffleButton,
				gridBag);
		gridBag.gridx = 2;
		buttonPanel.add(
				folderButton,
				gridBag);
		gridBag.gridx = 3;
		buttonPanel.add(
				loopButton,
				gridBag);
		gridBag.gridx = 4;
		buttonPanel.add(
				volumeButton,
				gridBag);

		gridBag.gridx = 0;
		gridBag.gridy = 2;
		gridBag.insets = new Insets(
				0,
				0,
				0,
				0);
		buttonPanel.add(
				previousButton,
				gridBag);
		gridBag.gridx = 1;
		buttonPanel.add(
				rewindButton,
				gridBag);
		gridBag.gridx = 2;
		buttonPanel.add(
				pauseButton,
				gridBag);
		gridBag.gridx = 3;
		buttonPanel.add(
				fastforwardButton,
				gridBag);
		gridBag.gridx = 4;
		buttonPanel.add(
				nextButton,
				gridBag);

		gridBag = new GridBagConstraints();
		gridBag.gridx = 0;
		gridBag.gridy = 3;
		gridBag.insets = new Insets(
				0,
				outsidePadding,
				outsidePadding,
				outsidePadding);
		this.getContentPane().add(
				buttonPanel,
				gridBag);
	}

	public JSlider getSeekSlider() {
		return seekSlider;
	}

	private void setKeyBindings() {
		for (Component comp : this.getContentPane().getComponents()) {
			JComponent jcomp = (JComponent) comp;

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("released LEFT"),
					"left");
			jcomp.getActionMap().put(
					"left",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							controller.previousMp3();
						}
					});

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("released DOWN"),
					"down");
			jcomp.getActionMap().put(
					"down",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							controller.rewindMp3();
						}
					});

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("released SPACE"),
					"space");
			jcomp.getActionMap().put(
					"space",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							controller.pauseMp3();
							buttonUpdate(
									pauseButton,
									"pause.png",
									"play.png");
						}
					});

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("UP"),
					"up");
			jcomp.getActionMap().put(
					"up",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							if (keyPressTime == null) {
								keyPressTime = System.currentTimeMillis();
							}
						}
					});

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("released UP"),
					"upgone");
			jcomp.getActionMap().put(
					"upgone",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							controller.fastForwardMp3(System.currentTimeMillis() - keyPressTime);
							keyPressTime = null;
						}
					});

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("released RIGHT"),
					"right");
			jcomp.getActionMap().put(
					"right",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							controller.nextMp3(new Exception(
									"Next mp3 was triggered."));
						}
					});

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("released 1"),
					"1");
			jcomp.getActionMap().put(
					"1",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							controller.relocateFile(0);
						}
					});

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("released 5"),
					"5");
			jcomp.getActionMap().put(
					"5",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							controller.relocateFile(1);
						}
					});

			jcomp.getInputMap(
					JComponent.WHEN_IN_FOCUSED_WINDOW).put(
					KeyStroke.getKeyStroke("released 9"),
					"9");
			jcomp.getActionMap().put(
					"9",
					new AbstractAction() {
						public void actionPerformed(
								ActionEvent e ) {
							controller.relocateFile(2);
						}
					});

		}
	}
}
