package mp3sorter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;
import java.util.Random;

import javax.swing.JFileChooser;

import javazoom.jlgui.basicplayer.BasicController;
import javazoom.jlgui.basicplayer.BasicPlayer;
import javazoom.jlgui.basicplayer.BasicPlayerEvent;
import javazoom.jlgui.basicplayer.BasicPlayerException;
import javazoom.jlgui.basicplayer.BasicPlayerListener;

public class Mp3PlayerController implements
		BasicPlayerListener
{
	private final MainWindow mainWindow;
	private BasicPlayer mp3Player = null;
	private Random randomGenerator = new Random();

	private Integer fileIndex;
	private ArrayList<File> fileList = new ArrayList<File>();
	private ArrayList<Integer> fileHistory = new ArrayList<Integer>();
	private int filesToCache = 200;

	private File directory = null;
	private String[] sortDirs;

	private boolean isShuffleEnabled = false;
	private boolean isLoopEnabled = false;
	private boolean isPaused = false;

	private long currentBytesRead = 0;
	private long totalBytes = 0;

	private int currentVolume = 100;

	public Mp3PlayerController(
			MainWindow mainWindow ) {
		this.mainWindow = mainWindow;
		readOptions();
	}

	private void readOptions() {
		String initialDirectory = "/";

		try {
			BufferedReader optionReader = new BufferedReader(
					new FileReader(
							new File(
									"./profile/options.txt")));
			while (optionReader.ready()) {
				if (optionReader.readLine().matches(
						"Initial Directory:")) {
					if (optionReader.ready()) {
						initialDirectory = optionReader.readLine();
					}
				}
			}
			optionReader.close();
		}
		catch (FileNotFoundException fne) {
			fne.printStackTrace();
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
		}

		sortDirs = new String[3];
		sortDirs[0] = "no";
		sortDirs[1] = "maybe";
		sortDirs[2] = "yes";

		setDirectory(initialDirectory);
		rebuildFileList();
		renewMp3Player();
	}

	private void writeOptions() {
		try {
			BufferedWriter optionWriter = new BufferedWriter(
					new FileWriter(
							new File(
									"./profile/options.txt")));

			optionWriter.write("Initial Directory:\n");
			optionWriter.write(directory.getAbsolutePath() + "\n");
			optionWriter.close();
		}
		catch (FileNotFoundException fne) {
			fne.printStackTrace();
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	private void setDirectory(
			String newLocation ) {
		directory = new File(
				newLocation);
		if (!directory.exists()) {
			directory = new File(
					"/");
		}

		writeOptions();
	}

	public synchronized void changeDir() {
		JFileChooser chooser = new JFileChooser();
		chooser.setCurrentDirectory(directory);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int returnVal = chooser.showOpenDialog(mainWindow);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			setDirectory(chooser.getSelectedFile().getAbsolutePath());
			rebuildFileList();
			renewMp3Player();
		}
	}

	private void rebuildFileList() {
		fileList.clear();
		fileHistory.clear();
		recurseIntoSubDirs(directory);
		fileIndex = 0;
	}

	private void recurseIntoSubDirs(
			File directory ) {
		File[] mp3FileList = directory.listFiles(new FilenameFilter() {
			public boolean accept(
					File dir,
					String name ) {
				return name.endsWith(".mp3");
			}
		});
		if (mp3FileList == null) {
			return;
		}
		for (File mp3File : mp3FileList) {
			fileList.add(mp3File);
		}

		File[] mp3DirList = directory.listFiles(new FileFilter() {
			public boolean accept(
					File pathname ) {
				return (pathname.isDirectory() && (!pathname.getName().startsWith(
						".")));
			}
		});
		if (mp3DirList == null) {
			return;
		}
		for (File mp3Dir : mp3DirList) {
			recurseIntoSubDirs(mp3Dir);
		}
	}

	private void renewMp3Player() {
		if (!fileList.isEmpty()) {
			try {
				if (mp3Player != null) {
					mp3Player.stop();
				}
				else {
					mp3Player = new BasicPlayer();
					mp3Player.addBasicPlayerListener(this);
				}
				mp3Player.open(fileList.get(fileIndex));
				mp3Player.play();
			}
			catch (BasicPlayerException e) {
				nextMp3(e);
			}
		}
	}

	private void decFileIndex() {
		if (!fileList.isEmpty()) {
			if (!fileHistory.isEmpty()) {
				fileIndex = fileHistory.get(fileHistory.size() - 1);
				fileHistory.remove(fileHistory.size() - 1);
				Integer oldFileIndex = fileIndex;
				if (checkFileIndex()) fileHistory.remove(oldFileIndex);
			}
			else {
				--fileIndex;
				checkFileIndex();
			}
		}
	}

	private void incFileIndex() {
		if (!fileList.isEmpty()) {
			fileHistory.add(fileIndex);
			if (fileHistory.size() > filesToCache) {
				fileHistory.remove(0);
			}

			if (isShuffleEnabled) {
				fileIndex = randomGenerator.nextInt(fileList.size());
			}
			else {
				++fileIndex;
				checkFileIndex();
			}
		}
	}

	public synchronized void relocateFile(
			int rating ) {
		if (!fileList.isEmpty()) {
			File filename = fileList.get(fileIndex);
			fileList.remove((int) fileIndex);
			fileHistory.remove(fileIndex);

			if (isShuffleEnabled) {
				fileIndex = randomGenerator.nextInt(fileList.size());
			}
			else {
				checkFileIndex();
			}
			renewMp3Player();

			String mp3Name = filename.getName();
			File mp3Dir = filename.getParentFile();
			ArrayList<String> subDirs = new ArrayList<String>();

			while (mp3Dir.compareTo(directory) != 0) {
				subDirs.add(
						0,
						mp3Dir.getName());
				mp3Dir = mp3Dir.getParentFile();
			}

			File newDir = new File(
					directory,
					"." + sortDirs[rating]);
			for (String subDir : subDirs) {
				newDir = new File(
						newDir,
						subDir);
			}
			newDir.mkdirs();

			File newLocation = new File(
					newDir,
					mp3Name);
			int copy = 2;
			while (newLocation.exists()) {
				newLocation = new File(
						newDir,
						mp3Name + " (" + copy + ")");
				++copy;
			}
			filename.renameTo(newLocation);
		}
	}

	private boolean checkFileIndex() {
		boolean changed = false;
		if (!fileList.isEmpty()) {
			if (fileIndex < 0) {
				fileIndex = fileList.size() - 1;
				changed = true;
			}
			else if (fileIndex >= fileList.size()) {
				fileIndex = 0;
				changed = true;
			}
		}
		return changed;
	}

	public synchronized void previousMp3() {
		decFileIndex();
		renewMp3Player();
	}

	public synchronized void rewindMp3() {
		renewMp3Player();
	}

	public synchronized void pauseMp3() {
		if (mp3Player != null) {
			try {
				if (isPaused) {

					mp3Player.resume();

					isPaused = false;
				}
				else {
					mp3Player.pause();
					isPaused = true;
				}
			}
			catch (BasicPlayerException e) {

			}
		}
	}

	public synchronized void fastForwardMp3(
			long bytesToSkip ) {
		if (mp3Player != null) {
			try {
				mp3Player.seek(currentBytesRead + 1000 * bytesToSkip);
			}
			catch (BasicPlayerException e) {

			}
		}
	}

	public synchronized void nextMp3(
			Exception e ) {
		if ((!isLoopEnabled) || (!e.getMessage().matches(
				"success"))) {
			incFileIndex();
		}

		renewMp3Player();
	}

	public synchronized void toggleShuffle() {
		isShuffleEnabled = !isShuffleEnabled;
	}

	public synchronized void toggleLoop() {
		isLoopEnabled = !isLoopEnabled;
	}

	public void opened(
			Object stream,
			Map properties ) {
		String author = "";
		String album = "";
		String title = "";
		if (properties != null) {
			if (properties.get("author") != null) author = properties.get(
					"author").toString().trim();
			if (properties.get("album") != null) album = properties.get(
					"album").toString().trim();
			if (properties.get("title") != null) title = properties.get(
					"title").toString().trim();
			if (properties.get("audio.length.bytes") != null) totalBytes = (Integer) properties.get("audio.length.bytes");
		}

		mainWindow.authorLabel.setText("Artist: " + author);
		mainWindow.albumLabel.setText("Album: " + album);

		if (title.matches("")) {
			title = fileList.get(
					fileIndex).getName();
			title = title.substring(
					0,
					title.length() - 4);
		}
		mainWindow.titleLabel.setText("Title: " + title);

		mainWindow.validate();
		mainWindow.pack();
	}

	public void progress(
			int bytesread,
			long microseconds,
			byte[] pcmdata,
			Map properties ) {
		currentBytesRead = bytesread;
		mainWindow.getSeekSlider().setValue(
				(int) ((1000 * currentBytesRead) / totalBytes));
	}

	public void setController(
			BasicController controller ) {

	}

	public void stateUpdated(
			BasicPlayerEvent event ) {
		if (event.getCode() == BasicPlayerEvent.EOM) {
			nextMp3(new Exception(
					"success"));
		}
		else if (event.getCode() == BasicPlayerEvent.OPENED) {
			setVolume(currentVolume);
		}
	}

	// Reads an integer from 1-100
	public synchronized void setVolume(
			int volume ) {
		try {
			currentVolume = volume;
			mp3Player.setGain(volume / 100.0);
		}
		catch (BasicPlayerException e) {

		}
	}

	// Reads an integer from 1-1000
	public synchronized void seekTo(
			int percentage ) {
		try {
			mp3Player.seek((totalBytes * percentage) / 1000);
		}
		catch (BasicPlayerException e) {

		}
	}
}
